<!doctype html>
<html lang="it-IT">

<head>
    <title>Account Registration</title>
</head>

<body>
    <p>Hi <strong>{{ user.email }},</strong></p>
    <p>This is your auth code: <span>{{ token }}</span></p>
</body>
</html>
