__version__ = "3.0.0.rc3"

__author_name__ = "cs91chris"
__author_email__ = "cs91chris@voidbrain.me"

__author__ = f"{__author_name__} <{__author_email__}>"

__all__ = [
    "__version__",
    "__author__",
    "__author_name__",
    "__author_email__",
]
